function [ x_hat ] = dct1( x )
%DCT1 Direct (slow) computation of one-dimensional DCT-I
%
%   Input parameter:
%      x - input coefficients
%
%    Ouput parameter:
%       x_hat - transformed coefficients
%
% Copyright (c) 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

x = x(:);
N = length(x);
x_hat = zeros(size(x));

if length(x) > 2
  n = (1:N-2)';
  for k=0:N-1
    x_hat(k+1) = (x(1) + (-1)^k * x(end))/2 + sum(x(n+1) .* cos(pi/(N-1) * k .* n));
  end
else
  for ik=1:length(x)
    x_hat(ik) = (x(1) + (-1)^(ik-1) * x(end))/2;
  end
end

end

