function [ freqtilde ] = indexset_mirror_all( freq )
%INDEXSET_MIRROR_ALL Mirrors all frequencies (repeatedly) at each
%coordinate axis.
%
%   Input parameters:
%      freq - frequency index set as matrix of size [num_freq,d]
%
%    Ouput parameters:
%      freqtilde - mirrored frequency index set
%
% Copyright (c) 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

  d = size(freq,2);
  m_all = [1;-1];
  for t=2:d
    m_all = [reshape(repmat([1,-1], size(m_all,1), 1), 2*size(m_all,1), 1) repmat(m_all, 2, 1)];
  end
  %index set \tilde{I} = \mathcal{M}(I)
  freqtilde = zeros(size(freq,1) * size(m_all,1),d);
  for im = 1:size(m_all,1)
    freqtilde((im-1)*size(freq,1)+1 : im*size(freq,1), :) = freq .* repmat(m_all(im,:), size(freq,1), 1);
  end
  freqtilde = unique(freqtilde,'rows');

end

