function [ results ] = test3_f9_sfft_cl_algorithm_multipletests( N_array, threshold_theta_array, algorithm_name, niter_r, nruns, filename )
%TEST3_F9_SFFT_CL_ALGORITHM_MULTIPLETESTS Runs multiple test sets of sfft_cl_algorithm.
%   This function may run many tests sets with a large amount of repititions.
%
%   Input parameters:
%      N_array - array of search space parameters N, Gamma = [0,..,N]^{10}
%      threshold_theta_array - array of corresponding relative thresholds theta
%      algorithm_name - type of algorithm,
%                       either 'a1cl' for Algorithm 1
%                       or 'a2cl' for Algorithm 2
%      niter_r - number of detection iterations r
%      nruns - number of repetition for one test setting
%      filename - output file name, where the output results are written to
%
%   Output parameter:
%      results - 3d array of type struct with size
%                [length(N_array), length(d_array), nruns]
%
% Copyright (c) 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

threshold_theta_basic = 1e-12;

if length(N_array) ~= length(threshold_theta_array)
  error('input N_array and threshold_theta_array must be of identical length');
end

if nargin < 6
  filename = 'test3_f9_results.mat';
end

results = repmat(struct(), length(threshold_theta_array), nruns);

for iN=1:length(N_array)
  N = N_array(iN);
  threshold_theta = threshold_theta_array(iN);
  for ir = 1:nruns
    [ num_freq_out, err_rel, err_rel_trunc, err_rel_alias, freq_cand_sizes, lattice_sizes, sample_sizes, z, run_times ] ...
      = test3_f9_sfft_cl_algorithm_singletest( N, algorithm_name, threshold_theta, threshold_theta_basic, niter_r );
    fprintf('run=%2d, N=%3d, threshold_theta=%.1e, theta_basic=%.1e, freq_out=%6d, err_rel=%.3e\n', ir, N, threshold_theta, threshold_theta_basic, num_freq_out, err_rel);
    %drawnow;
    results(iN,ir).N = N;
    results(iN,ir).threshold_theta = threshold_theta;
    results(iN,ir).num_freq_out = num_freq_out;
    results(iN,ir).err_rel = err_rel;
    results(iN,ir).err_rel_trunc = err_rel_trunc;
    results(iN,ir).err_rel_alias = err_rel_alias;
    results(iN,ir).freq_cand_sizes = freq_cand_sizes;
    results(iN,ir).lattice_sizes = lattice_sizes;
    results(iN,ir).sample_sizes = sample_sizes;
    results(iN,ir).z = z;
    results(iN,ir).run_times = run_times; % basic, lattice search, sampling, lfft, all
    save(filename, 'N_array', 'threshold_theta_array', 'algorithm_name', 'threshold_theta_basic', 'niter_r', 'nruns', 'results');
  end
end

end
